;*****************************************************************************
;
;                           
;                                    Mrslabor 2. 
;                                    Hzi Feladat
;
;*****************************************************************************
;
;*****************************************************************************
;* AVR mrpanel portkioszts:
;*****************************************************************************
;*
;* LED0(P):PortC.0          LED4(P):PortC.4
;* LED1(P):PortC.1          LED5(P):PortC.5
;* LED2(S):PortC.2          LED6(S):PortC.6
;* LED3(Z):PortC.3          LED7(Z):PortC.7        INT:PortE.4
;*
;* SW0:PortG.0     SW1:PortG.1     SW2:PortG.4     SW3:PortG.3
;* 
;* BT0:PortE.5     BT1:PortE.6     BT2:PortE.7     BT3:PortB.7
;*
;*****************************************************************************
;*
;* AIN:PortF.0     NTK:PortF.1    OPTO:PortF.2     POT:PortF.3
;*
;*****************************************************************************
;*
;* LCD1(VSS) = GND         LCD9(DB2): -
;* LCD2(VDD) = VCC         LCD10(DB3): -
;* LCD3(VO ) = GND         LCD11(DB4): PortA.4
;* LCD4(RS ) = PortA.0     LCD12(DB5): PortA.5
;* LCD5(R/W) = GND         LCD13(DB6): PortA.6
;* LCD6(E  ) = PortA.1     LCD14(007): PortA.7
;* LCD7(DB0) = -           LCD15(BLA): VCC
;* LCD8(DB1) = -           LCD16(BLK): PortB.5 (1=Httrvilgts ON)
;*
;*****************************************************************************

.nolist
.include "m128def.inc"    ; ATMega 128 defincis fjl 
.list

;***** Konstansok *****

.equ    tconst    = 25            ; idztsi konstans (T = 25*10 msec)

        ;***** Regiszterkioszts ***** 

.
.def    temp    = r16            ; lt. segdregiszter 
.def    ledek    = r17            ; ledek
.def    elozo    = r18            ; ledek bekapcsolva
.def    fenyero = r19
.def    elozoe  = r20
.def    honnan  = r21
.def    temp2 = r22
;*****************************************************************************
;* Reset & IT vektorok  
;*****************************************************************************
        
        .cseg 
        
        .org    0x0000        ; Kdszegmens kezdcme 
    
        jmp        RESET        ; Reset vektor 
        jmp        dummy        ; EXTINT0 Handler
        jmp        dummy        ; EXTINT1 Handler
        jmp        dummy        ; EXTINT2 Handler
        jmp        dummy        ; EXTINT3 Handler
        jmp        dummy  		; EXTINT4 Handler (INT gomb)
        jmp        dummy        ; EXTINT5 Handler 
        jmp        dummy        ; EXTINT6 Handler
        jmp        dummy        ; EXTINT7 Handler
        jmp        dummy        ; Timer2 Compare Match Handler 
        jmp        dummy        ; Timer2 Overflow Handler 
        jmp        dummy        ; Timer1 Capture Event Handler 
        jmp        dummy        ; Timer1 Compare Match A Handler 
        jmp        dummy        ; Timer1 Compare Match B Handler 
        jmp        dummy        ; Timer1 Overflow Handler 
        jmp        TIMER_COMPARE        ; Timer0 Compare Match Handler 
        jmp        TIMER_OVERFLOW        ; Timer0 Overflow Handler  
        jmp        dummy        ; SPI Transfer Complete Handler 
        jmp        dummy        ; USART0 RX Complete Handler 
        jmp        dummy        ; USART0 Data Register Empty Hanlder 
        jmp        dummy        ; USART0 TX Complete Handler 
        jmp        ADC_Complete_IT        ; ADC Conversion Complete Handler 
        jmp        dummy        ; EEPROM Ready Hanlder 
        jmp        dummy        ; Analog Comparator Handler 
        jmp        dummy        ; Timer1 Compare Match C Handler 
        jmp        dummy        ; Timer3 Capture Event Handler 
        jmp        dummy        ; Timer3 Compare Match A Handler 
        jmp        dummy        ; Timer3 Compare Match B Handler 
        jmp        dummy        ; Timer3 Compare Match C Handler 
        jmp        dummy        ; Timer3 Overflow Handler 
        jmp        dummy        ; USART1 RX Complete Handler 
        jmp        dummy        ; USART1 Data Register Empty Hanlder 
        jmp        dummy        ; USART1 TX Complete Handler 
        jmp        dummy        ; Two-wire Serial Interface Handler 
        jmp        dummy        ; Store Program Memory Ready Handler 
    

    .org    0x0046

;*****************************************************************************
;* Fprogram
;*****************************************************************************
RESET:

    ldi temp, LOW(RAMEND) ; RAMEND = "max address in RAM"
    out SPL, temp           ; RAMEND value in "m128def.inc" 
    ldi temp, HIGH(RAMEND) 
    out SPH, temp 

M_INIT:
;< ki- s bemenetek inicializlsa stb > 
    ldi honnan, 0x00
    ldi temp, 0xFF
    out DDRC, temp ; Ledek kimenetek
    ldi ledek, 0x00


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;            
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; inicializci
   ldi temp, 0xff
   sts PORTG, temp
   ldi temp, 0x00
   sts DDRG, temp


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;            TIMER BELLTSA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        ldi        temp,0b00000111
                    ;       0.......        ; FOC=0
                    ;       .0..1...        ; WGM=10 (CTC mod)
                    ;       ..00....        ; COM=00 (kimenet tiltva)
                    ;       .....111        ; CS0=111 (CLK/1024)

        out        TCCR0,temp            ; Timer 0 TCCR0 regiszter
        ldi        temp,20                ; 11059200Hz/1024 = 108*100
        out        OCR0,temp            ; Timer 0 OCR0 regiszter
        ldi        temp,0b00000011
                ;       000000..        ; Timer2,1 IT tiltva
                ;       ......1.        ; OCIE0=1
                ;       .......1        ; TOIE0=0
        out        TIMSK,temp            ; Timer IT Mask regiszter
        sei                            ; globlis IT engedlyezve

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;            ADC BELLTSA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        ldi temp, 0b11100010
        ;           11......... Az sszehasonltshoz a bels 2.56V referenciartk lesz hasznlva
        ;           ..1........ Balra igaztott bitek az ADC dupla regiszterben.
        ;           ......00010 2. pin, vagyis fnyrzkel kivlasztva
        out ADMUX, temp
        ldi temp, 0b11101100
        ;           1.......... ADEN: ADC Enable
        ;           .1......... ADSC: ADC Start Conversion
        ;           ..1........ ADFR: ADC Free Running Select
        ;           ...0....... ADIF: ADC Interrupt Flag
        ;           ....1...... ADIE: ADC Interrupt Enable (akarsz -e interrupot)
        ;           .....100... ADPS2:0: ADC Prescaler Select Bits (128-as oszt)
        out ADCSR, temp

        ldi temp,0b11111111 ; LED kimenet
        out DDRC,temp 




;*************************************************************** 
;* MAIN program, Endless loop part

M_LOOP:   ; Main loop, endless

    lds temp2, PING   ;kapcsolk beolvassa r20 regiszterbe
    ldi temp, 0b11100010  ; opto
    sbrc temp2, 1     ;SW2 kapcsol miatt kell, rejtlyes okokbl a PING4-re van ktve
    ldi temp, 0b11100011  ; poti
    out ADMUX, temp

jmp M_LOOP


;A mrt fnyert tltjk be a kompartorba, gy, mivel 
; invertl PWM-ben vagyunk minl kisebb rtket ad az 
; A/D konverter, annl hosszabb ideig fog vilgtani a
; LED.
;-----------------------------------------------------
SetPWM_Width:
    out OCR0, fenyero  ; TimerCompare rtknek belltsa
    ret
;-----------------------------------------------------    

; IT RUTUIN

TIMER_COMPARE:
        push    temp                ; segdregiszter mentse
        in        temp,SREG            ; sttusz mentse
        push    temp
        
        ldi temp, 0x00
        out PORTC, temp ; Ledek belltsa

        pop        temp                ; regiszterek visszalltsa
        out        SREG,temp
        pop        temp
reti


TIMER_OVERFLOW:
        push    temp                ; segdregiszter mentse
        in        temp,SREG            ; sttusz mentse
        push    temp
        
        ldi temp, 0xFF
        out PORTC, temp ; Ledek belltsa

        pop        temp                ; regiszterek visszalltsa
        out        SREG,temp
        pop        temp
reti

ADC_Complete_IT:
    mov elozo, fenyero   ; tlagolshoz elmentjk az elz fnyrtket
    in fenyero, ADCH     ; beolvassuk a fels bitjeit az ADC-nek
    lsr fenyero          ; 2-vek osztjuk
    lsr elozo            
    add fenyero, elozo   ; sszeadjuk ket    

    andi fenyero, 0b1111_0000 ; maszkoljuk az als 4 bitet

    call SetPWM_Width    ; PWM jel szlessgnek lltsa
reti


;*****************************************************************************
dummy: reti
